@@ -16,6 +16,10 @@ from shot_scraper.utils import filename_for_url, url_or_file_path
 BROWSERS = ("chromium", "firefox", "webkit", "chrome", "chrome-beta")
 
 
+def console_log(msg):
+    click.echo(msg, err=True)
+
+
 def browser_option(fn):
     click.option(
         "--browser",
@@ -32,6 +36,40 @@ def user_agent_option(fn):
     return fn
 
 
+def log_console_option(fn):
+    click.option("--log-console", is_flag=True, help="Write console.log() to stderr")(
+        fn
+    )
+    return fn
+
+
+def skip_fail_options(fn):
+    click.option("--skip", is_flag=True, help="Skip pages that return HTTP errors")(fn)
+    click.option(
+        "--fail",
+        is_flag=True,
+        help="Fail with an error code if a page returns an HTTP error",
+    )(fn)
+    return fn
+
+
+def skip_or_fail(response, skip, fail):
+    if skip and fail:
+        raise click.ClickException("--skip and --fail cannot be used together")
+    if str(response.status)[0] in ("4", "5"):
+        if skip:
+            click.echo(
+                "{} error for {}, skipping".format(response.status, response.url),
+                err=True,
+            )
+            # Exit with a 0 status code
+            raise SystemExit
+        elif fail:
+            raise click.ClickException(
+                "{} error for {}".format(response.status, response.url)
+            )
+
+
 def reduced_motion_option(fn):
     click.option(
         "--reduced-motion",
@@ -139,9 +177,11 @@ def cli():
     type=click.File("w"),
     help="Log details of all requests to this file",
 )
+@log_console_option
 @browser_option
 @user_agent_option
 @reduced_motion_option
+@skip_fail_options
 def shot(
     url,
     auth,
@@ -162,9 +202,12 @@ def shot(
     interactive,
     devtools,
     log_requests,
+    log_console,
     browser,
     user_agent,
     reduced_motion,
+    skip,
+    fail,
 ):
     """
     Take a single screenshot of a page or portion of a page.
@@ -242,6 +285,7 @@ def shot(
                     return_bytes=True,
                     use_existing_page=use_existing_page,
                     log_requests=log_requests,
+                    log_console=log_console,
                 )
                 sys.stdout.buffer.write(shot)
             else:
@@ -251,6 +295,9 @@ def shot(
                     shot,
                     use_existing_page=use_existing_page,
                     log_requests=log_requests,
+                    log_console=log_console,
+                    skip=skip,
+                    fail=fail,
                 )
         except TimeoutError as e:
             raise click.ClickException(str(e))
@@ -325,6 +372,8 @@ def _browser_context(
 @browser_option
 @user_agent_option
 @reduced_motion_option
+@log_console_option
+@skip_fail_options
 def multi(
     config,
     auth,
@@ -336,6 +385,9 @@ def multi(
     browser,
     user_agent,
     reduced_motion,
+    log_console,
+    skip,
+    fail,
 ):
     """
     Take multiple screenshots, defined by a YAML file
@@ -377,7 +429,13 @@ def multi(
             if outputs and shot.get("output") not in outputs:
                 continue
             try:
-                take_shot(context, shot)
+                take_shot(
+                    context,
+                    shot,
+                    log_console=log_console,
+                    skip=skip,
+                    fail=fail,
+                )
             except TimeoutError as e:
                 if fail_on_error:
                     raise click.ClickException(str(e))
@@ -407,7 +465,9 @@ def multi(
     type=int,
     help="Wait this many milliseconds before failing",
 )
-def accessibility(url, auth, output, javascript, timeout):
+@log_console_option
+@skip_fail_options
+def accessibility(url, auth, output, javascript, timeout, log_console, skip, fail):
     """
     Dump the Chromium accessibility tree for the specifed page
 
@@ -419,7 +479,10 @@ def accessibility(url, auth, output, javascript, timeout):
     with sync_playwright() as p:
         context, browser_obj = _browser_context(p, auth, timeout=timeout)
         page = context.new_page()
-        page.goto(url)
+        if log_console:
+            page.on("console", console_log)
+        response = page.goto(url)
+        skip_or_fail(response, skip, fail)
         if javascript:
             _evaluate_js(page, javascript)
         snapshot = page.accessibility.snapshot()
@@ -460,8 +523,21 @@ def accessibility(url, auth, output, javascript, timeout):
 @browser_option
 @user_agent_option
 @reduced_motion_option
+@log_console_option
+@skip_fail_options
 def javascript(
-    url, javascript, input, auth, output, raw, browser, user_agent, reduced_motion
+    url,
+    javascript,
+    input,
+    auth,
+    output,
+    raw,
+    browser,
+    user_agent,
+    reduced_motion,
+    log_console,
+    skip,
+    fail,
 ):
     """
     Execute JavaScript against the page and return the result as JSON
@@ -500,7 +576,10 @@ def javascript(
             reduced_motion=reduced_motion,
         )
         page = context.new_page()
-        page.goto(url)
+        if log_console:
+            page.on("console", console_log)
+        response = page.goto(url)
+        skip_or_fail(response, skip, fail)
         result = _evaluate_js(page, javascript)
         browser_obj.close()
     if raw:
@@ -560,6 +639,8 @@ def javascript(
     help="Scale of the webpage rendering",
 )
 @click.option("--print-background", is_flag=True, help="Print background graphics")
+@log_console_option
+@skip_fail_options
 def pdf(
     url,
     auth,
@@ -573,6 +654,9 @@ def pdf(
     height,
     scale,
     print_background,
+    log_console,
+    skip,
+    fail,
 ):
     """
     Create a PDF of the specified page
@@ -595,7 +679,10 @@ def pdf(
     with sync_playwright() as p:
         context, browser_obj = _browser_context(p, auth)
         page = context.new_page()
-        page.goto(url)
+        if log_console:
+            page.on("console", console_log)
+        response = page.goto(url)
+        skip_or_fail(response, skip, fail)
         if wait:
             time.sleep(wait / 1000)
         if javascript:
@@ -650,8 +737,10 @@ def pdf(
 @click.option(
     "--wait", type=int, help="Wait this many milliseconds before taking the snapshot"
 )
+@log_console_option
 @browser_option
 @user_agent_option
+@skip_fail_options
 def html(
     url,
     auth,
@@ -659,8 +748,11 @@ def html(
     javascript,
     selector,
     wait,
+    log_console,
     browser,
     user_agent,
+    skip,
+    fail,
 ):
     """
     Output the final HTML of the specified page
@@ -681,7 +773,10 @@ def html(
             p, auth, browser=browser, user_agent=user_agent
         )
         page = context.new_page()
-        page.goto(url)
+        if log_console:
+            page.on("console", console_log)
+        response = page.goto(url)
+        skip_or_fail(response, skip, fail)
         if wait:
             time.sleep(wait / 1000)
         if javascript:
@@ -736,7 +831,8 @@ def install(browser):
 @browser_option
 @user_agent_option
 @click.option("--devtools", is_flag=True, help="Open browser DevTools")
-def auth(url, context_file, browser, user_agent, devtools):
+@log_console_option
+def auth(url, context_file, browser, user_agent, devtools, log_console):
     """
     Open a browser so user can manually authenticate with the specified site,
     then save the resulting authentication context to a file.
@@ -756,6 +852,8 @@ def auth(url, context_file, browser, user_agent, devtools):
         )
         context = browser_obj.new_context()
         page = context.new_page()
+        if log_console:
+            page.on("console", console_log)
         page.goto(url)
         click.echo("Hit <enter> after you have signed in:", err=True)
         input()
@@ -770,10 +868,6 @@ def auth(url, context_file, browser, user_agent, devtools):
         pathlib.Path(context_file).chmod(0o600)
 
 
-class ShotError(Exception):
-    pass
-
-
 def _check_and_absolutize(filepath):
     path = pathlib.Path(filepath)
     if path.exists():
@@ -787,10 +881,16 @@ def take_shot(
     return_bytes=False,
     use_existing_page=False,
     log_requests=None,
+    log_console=False,
+    skip=False,
+    fail=False,
 ):
     url = shot.get("url") or ""
     if not url:
-        raise ShotError("url is required")
+        raise click.ClickException("url is required")
+
+    if skip and fail:
+        raise click.ClickException("--skip and --fail cannot be used together")
 
     url = url_or_file_path(url, file_exists=_check_and_absolutize)
 
@@ -843,6 +943,9 @@ def take_shot(
     else:
         page = context_or_page
 
+    if log_console:
+        page.on("console", console_log)
+
     viewport = {}
     full_page = True
     if shot.get("width") or shot.get("height"):
@@ -854,7 +957,16 @@ def take_shot(
         if shot.get("height"):
             full_page = False
 
-    page.goto(url)
+    response = page.goto(url)
+    # Check if page was a 404 or 500 or other error
+    if str(response.status)[0] in ("4", "5"):
+        if skip:
+            click.echo(
+                "{} error for {}, skipping".format(response.status, url), err=True
+            )
+            return
+        elif fail:
+            raise click.ClickException("{} error for {}".format(response.status, url))
 
     if wait:
         time.sleep(wait / 1000)
